# -*- coding: utf-8 -*-
##############################################################################
#
#    Email marketing click & read tracker
#    Copyright (C) 2011 Florent Aide, <florent.aide@gmail.com>
#    Copyright (C) 2011, 2015, 2018 XCG Consulting
#                                   (http://www.xcg-consulting.fr/)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from md5 import md5

from odoo import _, api, exceptions, fields as odoo_fields, models
from odoo.tools import config


class EmailTemplate(models.Model):
    """overrides the base email template to hook the URL tracker
    """

    _inherit = "mail.template"

    @api.multi
    def generate_email(self, res_ids, fields=None):
        """override generate_email() from our parent to substitute URLs
        """
        values = super(EmailTemplate, self).generate_email(
            res_ids, fields=fields
        )

        if self.env.context is None:
            context = dict()
        else:
            context = self.env.context.copy()

        if context.get("use_tracker", False):

            # here we need to use tracker so let's do it
            tracker_base = context.get("tracker_base")

            # here we add a hash that contains informations about our
            # business unit... this is necessary to be able to have
            # a tracker serve multiple business units
            hostname_bu = self.env["ir.config_parameter"].get_param(
                "bounced.hostname"
            )
            port_bu = self.env["ir.config_parameter"].get_param("bounced.port")
            # Try to guess the value
            # this will not work with the xmlrpcs interface…
            if not hostname_bu:
                hostname_bu = config.get("xmlrpc_interface")
            if not hostname_bu:
                hostname_bu = "localhost"
            if not port_bu:
                port_bu = config.get("xmlrpc_port")
            database_bu = self.env.cr.dbname
            id_bu = "%s:%s/%s" % (hostname_bu, port_bu, database_bu)
            tracker_base += "/" + md5(id_bu).hexdigest()

            template_model_name = ""
            partner_id = None

            obj = self.env[values["model"]]
            obj_rec = obj.browse(values["res_id"])
            if obj_rec:
                template_model_name = obj_rec.name_get()[0][1]
            if "partner_ids" in values and values["partner_ids"]:
                partner_ids = values["partner_ids"]
                if len(partner_ids) == 1:
                    partner_id = partner_ids[0]
                if len(partner_ids) > 1:
                    # TODO more than one? try the one from the campaign
                    field = (
                        self.env["marketing.campaign.activity"]
                        .browse(context["activity_id"])
                        .campaign_id.partner_field_id
                    )
                    if field and obj_rec:
                        partner_id = getattr(obj_rec, field.name).id
                        # if the parter is not in the list, grab the first
                        if partner_id not in partner_ids:
                            partner_id = partner_ids[0]
            else:
                # try the campaign defined attribute
                field = (
                    self.env["marketing.campaign.activity"]
                    .browse(context["activity_id"])
                    .campaign_id.partner_field_id
                )
                if field and obj_rec:
                    partner_id = getattr(obj_rec, field.name).id

            # if there is no email_to in the mail, take the partner one if any
            email_to = values["email_to"]
            if partner_id and not email_to:
                email_to = self.env["res.partner"].browse(partner_id).email

            activity = self.env["marketing.campaign.activity"].browse(
                context["activity_id"]
            )
            track_values = {
                "campaign_activity_id": context["activity_id"],
                "campaign_id": activity.campaign_id.id,
                "template_model_name": template_model_name,
                "partner_id": partner_id,
                "sent": odoo_fields.Datetime.now(),
                "email_to": email_to,
                "res_id": values["res_id"],
                "email_template_id": self.id,
            }
            # XXX there is no category field in track object
            if "category" in values:
                track_values["category"] = values["category"]
            track_model = self.env["marketing.campaign.tracker.track"]
            trackitem_model = self.env["marketing.campaign.tracker.trackitem"]
            track_id = track_model._create_batch(track_values)
            values["track_id"] = track_id

            new_body_text, tracks = track_model.insert_tracker_in_text(
                values.get("body", ""), tracker_base, track_id
            )

            for trackvalues in tracks:
                if "opt_out" not in obj.fields_get(["opt_out"]):
                    # check that opt_out field exists
                    if "opt_out" in obj.fields_get(["opt_out"]):
                        raise exceptions.Warning(
                            _('No opt_out field in the model "%s".')
                            % values["model"]
                        )
                # create a trackitem for each URL that is replaced in text
                trackvalues.update(
                    {
                        "campaign_activity_id": activity.id,
                        "campaign_id": activity.campaign_id.id,
                        "name_clicker": template_model_name,
                    }
                )
                trackitem_model._create_batch(trackvalues)
            values["body"] = new_body_text

            new_body_html, tracks = track_model.insert_tracker_in_html(
                values.get("body_html"), tracker_base, track_id
            )

            for trackvalues in tracks:

                if (
                    "callback" in trackvalues
                    and trackvalues["callback"]
                    in track_model.authorized_methods()
                ):

                    # check that opt_out field exists
                    if "opt_out" not in obj.fields_get(["opt_out"]):
                        raise exceptions.Warning(
                            _('No opt_out field in the model "%s".')
                            % values["model"]
                        )
                # create a trackitem for each URL that is replaced in HTML
                trackvalues.update(
                    {
                        "campaign_activity_id": activity.id,
                        "campaign_id": activity.campaign_id.id,
                        "name_clicker": template_model_name,
                    }
                )
                trackitem_model._create_batch(trackvalues)
            values["body_html"] = new_body_html

            # TODO do not hardcode
            # X-Mailgun-Tag could be used too
            values["headers"] = (
                "{'X-Mailgun-Tag':'%(campaign_id)s',"
                "'X-Mailgun-Track':'yes',"
                "'X-Mailgun-Track-Clicks':'no',"
                "'X-Mailgun-Variables':\"%(json)s\"}"
                % {
                    # TODO what would make sense there? name of the campaign?
                    "campaign_id": "%s-%d"
                    % (database_bu, activity.campaign_id.id),
                    "json": "{'activity_id': %(activity_id)d, }"
                    % {"activity_id": context["activity_id"]},
                }
            )
        return values


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
