# -*- coding: utf-8 -*-
##############################################################################
#
#    Email marketing click & read tracker
#    Copyright (C) 2011 Florent Aide, <florent.aide@gmail.com>
#    Copyright (C) 2011, 2015, 2018 XCG Consulting
#                                   (http://www.xcg-consulting.fr/)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from odoo import api, exceptions, fields as odoo_fields, models


class MarketingCampaignActivity(models.Model):
    """Change the marketing campaign activity to inject our new options
    """

    _inherit = "marketing.campaign.activity"

    # here we add a new field to the marketing campaign activity
    # the use_tracker boolean will activate the tracking for URLs if is
    # set to True. The default value will be False in order to avoid any
    # problem with existing activities previous to this module installation.
    use_tracker = odoo_fields.Boolean(
        string="Use Tracker",
        help='Activates the usage of an external "Bounced" click tracker',
        default=False,
    )
    tracker_base = odoo_fields.Char(
        string="Tracker Base URL",
        size=200,
        help="The base URL on which the tracker listens",
    )

    # make sure an activity which uses a tracker always has a tracker
    # base defined...
    @api.one
    @api.constrains("tracker_base", "use_tracker")
    def _check_baseurl(self):
        """Validates that every activity which has the use_tracker option set
        also has a tracker_base defined
        """
        if self.use_tracker and not self.tracker_base:
            return exceptions.ValidationError(
                "If you use tracking, you must give a tracker base URL"
            )

    # override the _process_wi_email to replace all the URLs in the template
    # body by our tracker URL and store the corresponding keys in database
    @api.multi
    def _process_wi_email(self, workitem):
        self.ensure_one()
        return super(
            MarketingCampaignActivity, self._update_context_with_tracker_info()
        )._process_wi_email(workitem)

    @api.multi
    def _update_context_with_tracker_info(self):
        result = self.with_context(use_tracker=self.use_tracker)

        if self.use_tracker:
            # remove any trailing slash that could be present in the base_url
            # and inject the necessary keys into our context
            result = result.with_context(
                tracker_base=self.tracker_base.rstrip("/"), activity_id=self.id
            )
        return result


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
