# -*- coding: utf-8 -*-
##############################################################################
#
#    Email marketing click & read tracker
#    Copyright (C) 2011 Florent Aide, <florent.aide@gmail.com>
#    Copyright (C) 2011, 2015 XCG Consulting (http://www.xcg-consulting.fr/)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import logging

from odoo import api, fields, models

from odoo.addons.marketing_campaign_tracker.replace import (
    insert_tracker_in_html,
    insert_tracker_in_text,
)

_logger = logging.getLogger(__name__)


class Track(models.Model):
    """For each mail sent/queued one of those will be created
    """

    _name = "marketing.campaign.tracker.track"
    _description = "Campaign Mail Tracker"
    _rec_name = "template_model_name"

    campaign_activity_id = fields.Many2one(
        comodel_name="marketing.campaign.activity",
        string="Marketing Campaign Activity",
        required=True,
        readonly=True,
        ondelete="cascade",
    )
    campaign_id = fields.Many2one(
        comodel_name="marketing.campaign",
        string="Marketing Campaign",
        related="campaign_activity_id.campaign_id",
        # XXX needed?
        store=True,
        readonly=True,
    )
    template_model_name = fields.Char(
        string="Name",
        help="Name of object from the mail template",
        # XXX give a size?
        readonly=True,
    )
    email_to = fields.Char(
        string="Email",
        # XXX give a size?
        readonly=True,
    )
    partner_id = fields.Many2one(
        comodel_name="res.partner", string="Partner", readonly=True
    )
    res_id = fields.Integer(string="Resource ID")
    message_uuid = fields.Char(string="Message-ID")
    scheduled = fields.Datetime(
        help="Date when the email has been created",
        default=fields.Datetime.now,
        readonly=True,
    )

    sent = fields.Datetime(
        string="Sent Date and time", help="Date and time sent from odoo"
    )
    opened = fields.Datetime(string="Opened")
    last_event_type = fields.Char(
        compute="_compute_last_event_type",
        help="Type of the most recent event, visit and open included",
        store=True,
    )
    event_ids = fields.One2many(
        comodel_name="marketing.campaign.tracker.track.event",
        inverse_name="track_id",
        string="Events",
    )
    item_ids = fields.One2many(
        comodel_name="marketing.campaign.tracker.trackitem",
        inverse_name="track_id",
        string="Items",
    )
    visit_ids = fields.One2many(
        comodel_name="marketing.campaign.tracker.trackvisit",
        inverse_name="track_id",
        string="Visits",
    )
    email_template_id = fields.Many2one("mail.template", "Email Template")

    @api.depends("event_ids.event_datetime", "visit_ids.visit_time", "opened")
    def _compute_last_event_type(self):
        for track in self:
            last_event = (
                track.event_ids[0] if len(track.event_ids) > 0 else None
            )
            last_visit = (
                track.visit_ids[0] if len(track.visit_ids) > 0 else None
            )
            if not last_event or track.opened > last_event.event_datetime:
                if not last_visit:
                    track.last_event_type = "open" if track.opened else None
                elif track.opened > last_visit.visit_time:
                    track.last_event_type = "open" if track.opened else None
                else:
                    track.last_event_type = "visit"
            else:
                if (
                    last_event
                    and last_visit
                    and last_event.event_datetime > last_visit.visit_time
                ):
                    track.last_event_type = last_event.type
                else:
                    track.last_event_type = "visit" if last_visit else None

    @api.model
    def insert_tracker_in_text(self, data, tracker_base, track_id):
        return insert_tracker_in_text(data, tracker_base, track_id)

    @api.model
    def insert_tracker_in_html(self, data, tracker_base, track_id):
        return insert_tracker_in_html(data, tracker_base, track_id)

    @api.model
    def authorized_methods(self):
        return ["opt_out"]

    @api.model
    def opt_out(self, track_id):
        """Change the resource opt out to True
        """
        track = self.browse(track_id)
        model_name = track.campaign_id.object_id.model
        res_id = track.res_id
        return self.env[model_name].browse(res_id).write({"opt_out": True})

    @api.model
    def set_opened(self, track_id):
        """Change the resource opened. Uses the lowest visit_time of
        track_visit where the trackitem has the set_opened callback.
        """

        track = self.browse(track_id)

        item_ids = [
            item.id
            for item in self.env[
                "marketing.campaign.tracker.trackitem"
            ].search(
                [("track_id", "=", track_id), ("callback", "=", "set_opened")]
            )
        ]
        trackvisit = self.env["marketing.campaign.tracker.trackvisit"].search(
            [("trackitem_id", "in", item_ids)], limit=1, order="visit_time ASC"
        )
        if trackvisit:
            track.opened = trackvisit.visit_time

    @api.model
    def _create_batch(self, vals):
        """If you add fields to this object, redefine this method! This include
        related or computed fields.
        This method does no security checks to speed up things.
        :param vals: values for the object. No one2many field is accepted.
        Scheduled is not accepted. Provide stored related values to avoid
        having to use browse to get the value (no check are made to see if the
        provided value is correct).
        :return: an id of the created record
        """

        if "visit_ids" in vals or "item_ids" in vals or "event_ids" in vals:
            raise NotImplementedError
        if "scheduled" in vals:
            raise NotImplementedError
        cr = self.env.cr
        # using get instead of pop is a faster, but the gain is less than 0.2ms
        campaign_activity_id = vals.pop("campaign_activity_id")
        opened = vals.pop("opened", None)
        d = dict(
            create_id=self.env.user.id,
            write_id=self.env.user.id,
            campaign_activity_id=campaign_activity_id,
            template_model_name=vals.pop("template_model_name", ""),
            email_to=vals.pop("email_to", ""),
            partner_id=vals.pop("partner_id", None),
            res_id=vals.pop("res_id", None),
            message_uuid=vals.pop("message_uuid", None),
            sent=vals.pop("sent", None),
            opened=opened,
            email_template_id=vals.pop("email_template_id", None),
            # related are not computed with recompute call
            campaign_id=(
                vals.pop("campaign_id")
                if "campaign_id" in vals
                else self.env["marketing.campaign.activity"]
                .browse(campaign_activity_id)
                .campaign_id.id,
            ),
            # self compute, this works because of no visits, no events
            last_event_type="open" if opened else "",
        )

        # do the same as create, warn if there is field we didn't expect
        if vals:
            _logger.warning("_create_batch includes unknown fields: %s", vals)
        cr.execute(
            "INSERT INTO marketing_campaign_tracker_track "
            "(create_uid, write_uid, create_date, write_date, "
            "campaign_activity_id, template_model_name, email_to, partner_id, "
            "res_id, message_uuid, scheduled, sent, opened, "
            "email_template_id, campaign_id, last_event_type) "
            "VALUES "
            "(%(create_id)s, %(write_id)s, "
            "(now() at time zone 'UTC'), (now() at time zone 'UTC'), "
            "%(campaign_activity_id)s, %(template_model_name)s, %(email_to)s, "
            "%(partner_id)s, %(res_id)s, %(message_uuid)s, "
            "(now() at time zone 'UTC'), %(sent)s, %(opened)s, "
            "%(email_template_id)s, %(campaign_id)s, %(last_event_type)s) "
            "RETURNING id",
            d,
        )
        created_id = cr.fetchone()[0]
        return created_id
