# -*- coding: utf-8 -*-
##############################################################################
#
#    Email marketing click & read tracker
#    Copyright (C) 2011 Florent Aide, <florent.aide@gmail.com>
#    Copyright (C) 2011, 2015 XCG Consulting (http://www.xcg-consulting.fr/)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import logging

from odoo import api, fields, models

_logger = logging.getLogger(__name__)


class TrackItem(models.Model):
    """a trackitem is a simple link between a real URL and a UUID URL that
    was sent in an email. When the person clicks on the UUID URL the tracker
    server will hit the database with the UUID to try and find the real URL
    and then redirect the visitor to the real URL he wants to see.
    During this process the tracker will be responsible to create trackvisit
    objects associated with the relevant trackitem.

    A trackitem is also directly linked to a campaign activity in order to be
    easily able to analyze by campaign the click/read ratios
    """

    _name = "marketing.campaign.tracker.trackitem"
    _description = "Track Item"
    _log_access = False

    _rec_name = "name_clicker"

    trackitem_uuid = fields.Char(
        string="uuid", size=36, required=True, index=True
    )
    real_url = fields.Char(string="Real URL", size=500)
    track_id = fields.Many2one(
        comodel_name="marketing.campaign.tracker.track",
        string="Marketing Campaign Track",
        required=True,
        ondelete="cascade",
        index=True,
    )
    campaign_activity_id = fields.Many2one(
        comodel_name="marketing.campaign.activity",
        string="Marketing Campaign Activity",
        related="track_id.campaign_activity_id",
        store=True,
        readonly=True,
    )
    campaign_id = fields.Many2one(
        comodel_name="marketing.campaign",
        string="Marketing Campaign",
        related="track_id.campaign_activity_id.campaign_id",
        # XXX needed?
        store=True,
        readonly=True,
        # without this the campaign can not be deleted for some reason
        ondelete="cascade",
    )
    track_visit_ids = fields.One2many(
        comodel_name="marketing.campaign.tracker.trackvisit",
        inverse_name="trackitem_id",
        string="Track Visit Id",
    )
    name_clicker = fields.Char(
        string="Name Clicker",
        size=255,
        related="track_id.template_model_name",
        store=True,
        readonly=True,
    )
    # callback will be used by bounced, it is a callback
    # IMPORTANT: the name of the values must be method of this object
    callback = fields.Selection(
        selection=[("opt_out", "Opt out"), ("set_opened", "Set opened")]
    )
    # display_id will be used by bounced, it will indicate a static_page to
    # display instead of redirecting
    display_id = fields.Many2one(
        comodel_name="static_page",
        string="Page to display instead of real URL",
    )
    type = fields.Selection(
        [("image", "Image"), ("link", "Link")], string="Type of tracking item"
    )
    category = fields.Char()

    @api.one
    def opt_out(self):
        """Callback to opt out.
        Just creates an opt out task in the queue.
        """
        # create a queue of those... do not directly change it there
        # TODO handle the case where the res_id has become null
        self.env["queue.task"].create(
            {
                "queue_id": self.env.ref(
                    "marketing_campaign_tracker.opt_out_queue"
                ).id,
                "args_interpolation": "{'track_id': %d}" % self.track_id,
            }
        )
        return True

    @api.one
    def set_opened(self):
        """Callback to set opened datetime.
        Just creates a set opened task in the queue.
        """
        # create a queue of those... do not directly change it there
        # TODO handle the case where the res_id has become null
        self.env["queue.task"].create(
            {
                "queue_id": self.env.ref(
                    "marketing_campaign_tracker.set_opened_queue"
                ).id,
                "args_interpolation": "{'track_id': %d}" % self.track_id,
            }
        )
        return True

    @api.multi
    def _create_batch(self, vals):
        """If you add fields to this object, redefine this method! This include
        related or computed fields.
        This method does no security checks to speed up things.
        :param vals: values for the object. No one2many field is accepted.
        Provide stored related values to avoid having to use browse to get the
        value (no check are made to see if the provided value is correct).
        :return: an id of the created record
        """
        cr = self.env.cr

        params = self._create_batch_params(vals)
        # do the same as create, warn if there is field we didn't expect
        if vals:
            _logger.warning("_create_batch includes unknown fields: %s", vals)
        query = self._create_batch_build_query(params)
        cr.execute(query, params)
        created_id = cr.fetchone()[0]
        return created_id

    @api.model
    def _create_batch_params(self, vals):
        """Build a dictionary of parameters from vals. This method will add
        related if they are not present, and also computed fields.
        The vals dict will be emptied, any thing left is considered an unknown
        field.
        :param vals: values for the creation
        :type vals: dict
        :return: a dictionary of column and their values
        """
        if "track_visit_ids" in vals:
            raise NotImplementedError
        track_id = vals.pop("track_id")
        return dict(
            trackitem_uuid=vals.pop("trackitem_uuid"),
            real_url=vals.pop("real_url", ""),
            track_id=track_id,
            callback=vals.pop("callback", ""),
            display_id=vals.pop("display_id", None),
            type=vals.pop("type", ""),
            category=vals.pop("category", ""),
            # related are not computed with recompute call
            name_clicker=(
                vals.pop("name_clicker")
                if "name_clicker" in vals
                else self.env["marketing.campaign.tracker.track"]
                .browse(track_id)
                .template_model_name,
            ),
            campaign_activity_id=(
                vals.pop("campaign_activity_id")
                if "campaign_activity_id" in vals
                else self.env["marketing.campaign.tracker.track"]
                .browse(track_id)
                .campaign_activity_id.id,
            ),
            campaign_id=(
                vals.pop("campaign_id")
                if "campaign_id" in vals
                else self.env["marketing.campaign.tracker.track"]
                .browse(track_id)
                .campaign_activity_id.campaign_id.id,
            ),
        )

    def _create_batch_build_query(self, params):
        """Build an insert column from the provided list of columns/value.
        :param params: columns and their values
        :type params: dict
        :return: a parameterized request
        """
        keys = params.keys()
        return "INSERT INTO %s (%s) VALUES (%s) RETURNING id" % (
            self._table,
            ", ".join(keys),
            ", ".join("%({})s".format(key) for key in keys),
        )
